# -*- coding: utf-8 -*-
from flask import request, make_response, send_file
from flask_cors import CORS

from domain_admin.model import db
from domain_admin.router import routes
from domain_admin.utils.flask_ext.flask_app import FlaskApp
from domain_admin.utils.flask_ext.register import register_app_routers, register_error_handler

app = FlaskApp(__name__, static_folder="public", static_url_path="/")

CORS(app, supports_credentials=True)

# 路由
register_app_routers(app, routes)

# 异常处理
register_error_handler(app)


@app.before_request
def before_request():
    """跨域请求会出现options，直接返回即可"""
    if request.method == 'OPTIONS':
        return make_response()

    db.connect()


@app.teardown_request
def teardown_request(exc):
    if not db.is_closed():
        db.close()


@app.get('/')
def index():
    return send_file('public/index.html')



# -*- coding: utf-8 -*-
from flask import request

from domain_admin.utils import cert_util
from domain_admin.utils.flask_ext.app_exception import AppException


def get_cert_information():
    """
    获取域名证书信息
    :return:
    """
    if request.method == 'GET':
        domain = request.args.get('domain')
    else:
        domain = request.json.get('domain')

    if not domain:
        raise AppException('参数缺失：domain')

    try:
        data = cert_util.get_cert_info(domain)
    except Exception as e:
        data = None

    return data



# -*- coding: utf-8 -*-


def list_with_relation_one(lst, field, model):
    """
    获取一对一 关联对象
    :param lst: list
    :param field: 字段名称
    :param model: 模型对象
    :return: list
    """

    field_id = field + '_id'
    field_ids = set(map(lambda item: item[field_id], lst))

    field_models = model.select().where(
        model.id.in_(field_ids)
    )

    field_map = {}
    for field_model in field_models:
        field_map[field_model.id] = field_model

    for row in lst:
        row[field] = field_map.get(row[field_id])

    return lst


def list_with_relation_many(lst, field, model, limit=None):
    """待完成"""
    pass

    # field_id = field + '_id'
    # field_ids = set(map(lambda item: item['id'], lst))
    #
    # query = model.select().where(
    #     model.id.in_(field_ids)
    # )
    #
    # if limit:
    #     query = query.limit(limit)
    #
    # field_models = query
    #
    # field_map = {}
    # for field_model in field_models:
    #     field_model_list = field_map.get(field_model.id, [])
    #
    #     field_map[field_model.id] = field_model_list
    #
    # for row in lst:
    #     row[field] = field_map.get(row[field_id])
    #
    # return lst



# -*- coding: utf-8 -*-
import json
from functools import cmp_to_key
from operator import itemgetter

from playhouse.shortcuts import model_to_dict

from domain_admin.config import BEFORE_EXPIRE_DAYS, MAIL_TO_ADDRESSES
from domain_admin.model import DomainModel, GroupModel
from domain_admin.service import email_service, render_service
from domain_admin.utils import cert_util, datetime_util
from domain_admin.utils.cert_util import get_cert_info
from domain_admin.utils.datetime_util import get_datetime
from domain_admin.utils.peewee_ext.model_util import list_with_relation_one


def add_domain(data):
    """
    添加域名
    :param data: {
        domain 必传
        alias 可选，默认 ""
        group_id 可选，默认 0
     }
    :return:
    """
    domain = data['domain']
    alias = data.get('alias', '')
    group_id = data.get('group_id', 0)

    connect_status = False

    info = {}

    try:
        info = cert_util.get_cert_info(domain)
        connect_status = True
    except Exception:
        pass

    row = DomainModel.create(
        domain=domain,
        alias=alias,
        group_id=group_id,
        start_time=info.get('start_date'),
        expire_time=info.get('expire_date'),
        ip=info.get('ip'),
        connect_status=connect_status,
        detail_raw=json.dumps(info, ensure_ascii=False),
        check_time=datetime_util.get_datetime(),
    )

    return row


def update_domain_cert_info(row):
    """
    更新域名的证书信息
    :param row:
    :return:
    """

    connect_status = False

    info = {}

    try:
        info = get_cert_info(row.domain)
        connect_status = True
    except Exception:
        pass

    DomainModel.update(
        start_time=info.get('start_date'),
        expire_time=info.get('expire_date'),
        ip=info.get('ip'),
        connect_status=connect_status,
        detail_raw=json.dumps(info, ensure_ascii=False),
        check_time=get_datetime(),
    ).where(
        DomainModel.id == row.id
    ).execute()


def update_all_domain_cert_info():
    """
    更新所有域名信息
    :return:
    """
    lst = DomainModel.select()
    for row in lst:
        update_domain_cert_info(row)


def get_domain_info_list():
    lst = DomainModel.select()

    lst = list(map(lambda m: model_to_dict(
        model=m,
        exclude=[DomainModel.detail_raw],
        extra_attrs=[
            'start_date',
            'expire_date',
            'expire_days',
        ]
    ), lst))

    def compare(a, b):
        if a['expire_days'] and b['expire_days']:
            return a['expire_days'] - b['expire_days']
        else:
            if a['expire_days']:
                return a['expire_days']
            else:
                return -b['expire_days']

    lst = sorted(lst, key=cmp_to_key(compare))

    return lst


def check_domain_cert():
    """
    查询域名证书到期情况
    :return:
    """
    lst = get_domain_info_list()

    has_expired_domain = False

    for item in lst:
        if item['expire_days'] is None or item['expire_days'] <= BEFORE_EXPIRE_DAYS:
            has_expired_domain = True
            break

    if has_expired_domain:
        send_domain_list_email(MAIL_TO_ADDRESSES)


def send_domain_list_email(to_addresses):
    """
    发送域名信息
    :param to_addresses:
    :return:
    """
    lst = get_domain_info_list()

    content = render_service.render_template('domain-cert-email.html', {'list': lst})

    email_service.send_email(
        subject='[ssl]证书过期时间汇总',
        content=content,
        to_addresses=to_addresses,
        content_type='html'
    )


if __name__ == '__main__':
    print(get_domain_info_list())



# -*- coding: utf-8 -*-

from collections import Iterator

from flask import Flask
from peewee import ModelSelect, Model
from playhouse.shortcuts import model_to_dict

from domain_admin.utils.flask_ext.api_result import ApiResult
from domain_admin.utils.flask_ext.json.json_encoder import JSONEncoder
from domain_admin.utils.flask_ext.json.json_provider import JSONProvider
from domain_admin.utils.flask_ext.request import Request


class FlaskApp(Flask):
    """
    扩展Flask
    """
    # Flask <=2.0.0
    json_encoder = JSONEncoder

    # Flask > 2.0.0
    json_provider_class = JSONProvider

    request_class = Request

    def make_response(self, rv):

        if isinstance(rv, ModelSelect):
            rv = list(rv.dicts())

        if isinstance(rv, Model):
            rv = model_to_dict(rv)

        if isinstance(rv, Iterator):
            rv = list(rv)

        if isinstance(rv, (list, dict, int, str)) or rv is None:
            rv = ApiResult.success(rv)

        if isinstance(rv, ApiResult):
            rv = rv.to_dict()

        return super().make_response(rv)



# -*- coding: utf-8 -*-
"""
路由配置
"""
from domain_admin.api import cert_api
from domain_admin.api import domain_api
from domain_admin.api import group_api

routes = {
    "/api/getCertInformation": cert_api.get_cert_information,

    "/api/addDomain": domain_api.add_domain,
    "/api/updateDomainById": domain_api.update_domain_by_id,
    "/api/deleteDomainById": domain_api.delete_domain_by_id,
    "/api/getDomainList": domain_api.get_domain_list,
    "/api/getDomainById": domain_api.get_domain_by_id,
    "/api/updateDomainCertInfoById": domain_api.update_domain_cert_info_by_id,
    "/api/updateAllDomainCertInfo": domain_api.update_all_domain_cert_info,
    "/api/sendDomainInfoListEmail": domain_api.send_domain_info_list_email,
    "/api/checkDomainCert": domain_api.check_domain_cert,

    "/api/addGroup": group_api.add_group,
    "/api/updateGroupById": group_api.update_group_by_id,
    "/api/deleteGroupById": group_api.delete_group_by_id,
    "/api/getGroupList": group_api.get_group_list,
    "/api/getGroupById": group_api.get_group_by_id,

}



# -*- coding: utf-8 -*-
from .group_model import GroupModel
from .base_model import db
from .domain_model import DomainModel

tables = [
    DomainModel,
    GroupModel,
]

for table in tables:
    if not table.table_exists():
        table.create_table()



# -*- coding: utf-8 -*-


class ApiResult(object):
    """返回统一的数据结构"""

    def __init__(self, data, msg, code):
        self.data = data
        self.msg = msg
        self.code = code

    @classmethod
    def success(cls, data=None, msg='success', code=0):
        return cls(data, msg, code)

    @classmethod
    def error(cls, data=None, msg='error', code=-1):
        return cls(data, msg, code)

    def to_dict(self):
        return {
            'data': self.data,
            'code': self.code,
            'msg': self.msg
        }



# -*- coding: utf-8 -*-
from datetime import datetime

from domain_admin.model.base_model import BaseModel
from peewee import CharField, IntegerField, DateTimeField, BooleanField


class GroupModel(BaseModel):
    """分组"""
    id = IntegerField(primary_key=True)

    # 名称
    name = CharField()

    # 创建时间
    create_time = DateTimeField(default=datetime.now)

    # 更新时间
    update_time = DateTimeField(default=datetime.now)



# -*- coding: utf-8 -*-
from flask import Request as _Request


class Request(_Request):
    @property
    def json(self):
        data = self.get_json()

        if not data:
            data = {}

        return data



# -*- coding: utf-8 -*-

from operator import itemgetter

from playhouse.shortcuts import model_to_dict

from domain_admin.config import MAIL_HOST, MAIL_PORT, MAIL_USERNAME, MAIL_PASSWORD
from domain_admin.model import DomainModel
from domain_admin.service import render_service
from domain_admin.utils.email_util import EmailServer


def send_email(subject,
               content,
               to_addresses,
               content_type='plain'):
    email_server = EmailServer(
        mail_host=MAIL_HOST,
        mail_port=MAIL_PORT,
        mail_username=MAIL_USERNAME,
        mail_password=MAIL_PASSWORD
    )

    email_server.send_email(
        subject=subject,
        content=content,
        to_addresses=to_addresses,
        content_type=content_type
    )

    email_server.quit()


# -*- coding: utf-8 -*-
from peewee import DoesNotExist

from domain_admin.utils.flask_ext.api_result import ApiResult
import traceback


def error_handler(e):
    traceback.print_exc()

    if isinstance(e, KeyError):
        msg = '参数缺失' + str(e)

    elif isinstance(e, DoesNotExist):
        msg = '数据不存在'

    else:
        msg = str(e)

    return ApiResult.error(msg=msg)



# -*- coding: utf-8 -*-

import smtplib
from email.mime.text import MIMEText


class EmailServer(object):
    def __init__(self, mail_host, mail_port, mail_username, mail_password):
        self.mail_host = mail_host
        self.mail_port = mail_port
        self.mail_username = mail_username
        self.mail_password = mail_password
        self._server = None

    # def __del__(self):
    #     self.quit()

    @property
    def server(self):
        if self._server is None:
            self._server = self.get_server()

        return self._server

    def get_server(self):
        """
        获取邮件服务器
        """
        if self.mail_port == 465:
            # ssl 465端口
            server = smtplib.SMTP_SSL(self.mail_host)
        else:
            # 25端口
            server = smtplib.SMTP(self.mail_host, self.mail_port)

        # server.connect(host=self.mail_host, port=self.mail_port)

        # print(self.mail_username, self.mail_password)

        server.login(self.mail_username, self.mail_password)

        return server

    def quit(self):
        if self.server:
            self.server.quit()

    def get_email_content(self, subject, content, content_type='plain'):
        # 构造邮件
        msg = MIMEText(content, content_type, 'utf-8')
        # 邮箱昵称、发件人邮箱账号
        # msg['From'] = formataddr([self.mail_username, self.mail_username])
        # msg['To'] = to_addresses.join(',')
        msg['Subject'] = subject
        return msg

    def send_email(self, subject, content, to_addresses, content_type='plain'):
        """
        :param subject:
        :param content:
        :param to_addresses:
        :param content_type:  plain/html
        :return:
        """

        email_content = self.get_email_content(subject, content, content_type)

        # 发送邮件
        self.server.sendmail(
            from_addr=self.mail_username,
            to_addrs=to_addresses,
            msg=email_content.as_string()
        )



# -*- coding: utf-8 -*-
from datetime import datetime

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"

DATE_FORMAT = "%Y-%m-%d"


def get_datetime():
    return datetime.now().strftime(DATETIME_FORMAT)


def get_date():
    return datetime.now().strftime(DATE_FORMAT)


def format_datetime(date_time):
    return datetime.strftime(date_time, DATETIME_FORMAT)


def format_date(date_time):
    return datetime.strftime(date_time, DATE_FORMAT)



# -*- coding: utf-8 -*-
from domain_admin.utils.flask_ext.handler import error_handler


def register_app_routers(app, routers):
    """注册路由函数"""
    for url, func in routers.items():
        app.add_url_rule(url, None, func, methods=['POST', 'GET'])


def register_error_handler(app):
    """
    全局异常捕获，也相当于一个视图函数
    """
    app.register_error_handler(Exception, error_handler)



# -*- coding: utf-8 -*-

#################################
# 读取用户自定义变量
#################################

import os

import yaml

config_file = os.path.join(os.getcwd(), 'config.yml')

# logger.info('config_file: %s', config_file)

if os.path.exists(config_file):
    f = open(config_file, "rb")
    config = yaml.safe_load(f)
    f.close()
else:
    config = {}

##############################
#            邮箱配置
##############################
# 服务器地址
MAIL_HOST = config.get('MAIL_HOST')
# 服务器端口 25 或者 465(ssl)
MAIL_PORT = config.get('MAIL_PORT')

# 发件人邮箱账号
MAIL_USERNAME = config.get('MAIL_USERNAME')
# 发件人邮箱密码
MAIL_PASSWORD = config.get('MAIL_PASSWORD')

# 到期前几天邮件提醒
BEFORE_EXPIRE_DAYS = config.get('BEFORE_EXPIRE_DAYS')

# 提醒邮箱
MAIL_TO_ADDRESSES = config.get('MAIL_TO_ADDRESSES')

# 地址
FLASK_HOST = config.get('FLASK_HOST')

# 端口
FLASK_PORT = config.get('FLASK_PORT')


# -*- coding: utf-8 -*-

from flask import request
from playhouse.shortcuts import model_to_dict

from domain_admin.model import DomainModel, GroupModel
from domain_admin.service import domain_service
from domain_admin.utils import datetime_util
from domain_admin.utils.flask_ext.app_exception import AppException
from domain_admin.utils.peewee_ext import model_util


def add_domain():
    """
    添加域名
    :return:
    """
    domain = request.json.get('domain')
    alias = request.json.get('alias', '')
    group_id = request.json.get('group_id', 0)

    if not domain:
        raise AppException('参数缺失：domain')

    row = domain_service.add_domain({
        'domain': domain,
        'alias': alias,
        'group_id': group_id,
    })

    return {'id': row.id}


def update_domain_by_id():
    """
    更新数据
    id domain alias group_id notify_status
    :return:
    """

    data = request.get_json(force=True)
    domain_id = data.pop('id')

    data['update_time'] = datetime_util.get_datetime()

    DomainModel.update(data).where(
        DomainModel.id == domain_id
    ).execute()


def delete_domain_by_id():
    """
    删除
    :return:
    """
    domain_id = request.json.get('id')

    DomainModel.delete_by_id(domain_id)


def get_domain_list():
    """
    获取域名列表
    :return:
    """
    page = request.json.get('page', 1)
    size = request.json.get('size', 10)
    group_id = request.json.get('group_id')

    query = DomainModel.select()

    if isinstance(group_id, int):
        query = query.where(DomainModel.group_id == group_id)

    lst = query.order_by(
        DomainModel.create_time.asc()
    ).paginate(page, size)

    total = DomainModel.select().count()

    # TODO: N+1 解决问题
    lst = list(map(lambda m: model_to_dict(
        model=m,
        exclude=[DomainModel.detail_raw],
        extra_attrs=[
            'total_days',
            'expire_days',
        ]
    ), lst))

    # append_field(lst, ['group'])

    # domain_service.domain_list_with_group(lst)
    lst = model_util.list_with_relation_one(lst, 'group', GroupModel)

    return {
        'list': lst,
        'total': total
    }


def get_domain_by_id():
    """
    获取
    :return:
    """
    domain_id = request.json['id']

    row = DomainModel.get_by_id(domain_id)

    return model_to_dict(
        model=row,
        exclude=[DomainModel.detail_raw],
        extra_attrs=[
            'total_days',
            'expire_days',
            'detail',
            'group',
        ]
    )


def update_all_domain_cert_info():
    """
    更新所有域名证书信息
    :return:
    """
    domain_service.update_all_domain_cert_info()


def update_domain_cert_info_by_id():
    """
    更新域名证书信息
    :return:
    """
    domain_id = request.json['id']

    row = DomainModel.get_by_id(domain_id)

    domain_service.update_domain_cert_info(row)


def send_domain_info_list_email():
    """
    发送域名证书信息到邮箱
    :return:
    """
    to_addresses = request.json['to_addresses']
    domain_service.send_domain_list_email(to_addresses)


def check_domain_cert():
    """
    检查域名证书信息
    :return:
    """
    # 先更新，再检查
    domain_service.update_all_domain_cert_info()

    domain_service.check_domain_cert()



# -*- coding: utf-8 -*-
"""
生产环境启动入口
"""

from werkzeug.middleware.proxy_fix import ProxyFix

from domain_admin.app import app
from domain_admin.config import FLASK_HOST, FLASK_PORT

app.wsgi_app = ProxyFix(app.wsgi_app)

if __name__ == '__main__':
    app.run(port=FLASK_PORT, host=FLASK_HOST)



# -*- coding: utf-8 -*-

from jinja2 import Environment
from jinja2 import FileSystemLoader

from domain_admin.config import TEMPLATE_DIR


def render_template(template, data):
    """
    渲染模板
    :param template:
    :param data:
    :return:
    """
    env = Environment(loader=FileSystemLoader(TEMPLATE_DIR))
    t = env.get_template(template)
    return t.render(data)



# -*- coding: utf-8 -*-
from datetime import datetime

from peewee import ModelSelect, Model
from playhouse.shortcuts import model_to_dict

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"


def default_json_encoder(o):
    """
    json 序列化
    :param o:
    :return:
    """
    if isinstance(o, ModelSelect):
        return list(o.dicts())

    if isinstance(o, Model):
        return model_to_dict(o)

    if isinstance(o, datetime):
        return o.strftime(DATETIME_FORMAT)

    return o



# -*- coding: utf-8 -*-

import os

# 配置文件目录
CONFIG_DIR = os.path.dirname(__file__)

# 应用文件目录
APP_DIR = os.path.dirname(CONFIG_DIR)

# 项目根目录
ROOT_DIR = os.path.dirname(APP_DIR)

# 模板目录
TEMPLATE_DIR = os.path.join(APP_DIR, 'templates')

# 公开资源目录
PUBLIC_DIR = os.path.join(APP_DIR, 'public')



# -*- coding: utf-8 -*-
from flask import request

from domain_admin.model import GroupModel


def add_group():
    """
    添加
    :return:
    """
    name = request.json.get('name')

    row = GroupModel.create(
        name=name
    )

    return {'id': row.id}


def update_group_by_id():
    """
    更新数据
    :return:
    """
    group_id = request.json.get('id')
    name = request.json.get('name')

    GroupModel.update(
        name=name,
    ).where(
        GroupModel.id == group_id
    ).execute()


def delete_group_by_id():
    """
    删除
    :return:
    """
    group_id = request.json.get('id')

    GroupModel.delete_by_id(group_id)


def get_group_list():
    """
    获取域名列表
    :return:
    """
    # page = request.json.get('page', 1)
    # size = request.json.get('size', 10)

    lst = GroupModel.select().order_by(
        GroupModel.create_time.asc()
    )

    total = GroupModel.select().count()

    return {
        'list': lst,
        'total': total
    }


def get_group_by_id():
    """
    获取
    :return:
    """
    group_id = request.json.get('id')

    return GroupModel.get_by_id(group_id)



# -*- coding: utf-8 -*-
from peewee import Model
from playhouse.db_url import connect
# from playhouse.signals import Model, pre_save
from playhouse.sqliteq import SqliteQueueDatabase

from domain_admin.config import SQLITE_DATABASE_URL, SQLITE_DATABASE_PATH

import logging

# 打印日志
logger = logging.getLogger('peewee')
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)

# db = connect(SQLITE_DATABASE_URL)
db = SqliteQueueDatabase(database=SQLITE_DATABASE_PATH)


class BaseModel(Model):
    class Meta:
        database = db


# -*- coding: utf-8 -*-

# @Date    : 2019-06-26
# @Author  : Peng Shiyu
import glob
import io
import os

from setuptools import setup, find_packages

"""
## 本地测试
安装测试
python setup.py develop 
python setup.py develop --uninstall

卸载
pip uninstall spideradmin -y


## 打包上传
先升级打包工具
pip install --upgrade setuptools wheel twine

打包
python setup.py sdist bdist_wheel

检查
twine check dist/*

上传pypi
twine upload dist/*

命令整合
rm -rf dist build *.egg-info \
&& python setup.py sdist bdist_wheel  \
&& twine check dist/* \
&& twine upload dist/*


## 下载测试
安装测试
pip3 install -U spider-admin-pro -i https://pypi.org/simple

打包的用的setup必须引入

参考：
https://packaging.python.org/guides/making-a-pypi-friendly-readme/

"""

base_dir = os.path.dirname(os.path.abspath(__file__))

# 版本号
version_file = glob.glob("*/version.py", recursive=True)[0]

with io.open(version_file, 'rb') as f:
    version_var = {}
    exec(f.read(), version_var)
    VERSION = version_var['VERSION']

with io.open("README.md", 'r', encoding='utf-8') as f:
    long_description = f.read()

with io.open("requirements.txt", 'r') as f:
    install_requires = f.read().split(os.sep)

setup(
    name='domain-admin',
    version=VERSION,
    description="a domain ssl cert admin",

    keywords='domain ssl cert',
    author='Peng Shiyu',
    author_email='pengshiyuyx@gmail.com',
    license='MIT',
    url="https://github.com/mouday/domain-admin",

    long_description=long_description,
    long_description_content_type='text/markdown',

    # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7"
    ],

    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=install_requires
)



# -*- coding: utf-8 -*-

from domain_admin.utils.flask_ext.json.default import default_json_encoder

try:
    from flask.json.provider import DefaultJSONProvider, _default
except ImportError:
    class DefaultJSONProvider(object):
        pass


    def _default(o):
        pass


class JSONProvider(DefaultJSONProvider):
    """
    Flask 2.2.2
    """

    @staticmethod
    def default(o):
        ret = default_json_encoder(o)
        if ret is not o:
            return ret

        return _default(o)



# -*- coding: utf-8 -*-
from playhouse.migrate import SqliteMigrator, migrate

from domain_admin.model import db, DomainModel

if __name__ == '__main__':
    migrator = SqliteMigrator(db)

    migrate(
        # 2022-09-27
        # migrator.add_column(DomainModel._meta.table_name, DomainModel.notify_status.name, DomainModel.notify_status),
        # migrator.add_column(DomainModel._meta.table_name, DomainModel.ip.name, DomainModel.ip),
    )



# -*- coding: utf-8 -*-
from .default_config import *
from .yaml_config import *

# 'sqlite:///default.db'
SQLITE_DATABASE_PATH = os.path.join(ROOT_DIR, 'database.db')
SQLITE_DATABASE_URL = 'sqlite:///' + SQLITE_DATABASE_PATH



# -*- coding: utf-8 -*-
import json
from datetime import datetime
from domain_admin.model import GroupModel
from domain_admin.model.base_model import BaseModel
from peewee import CharField, IntegerField, DateTimeField, BooleanField, TextField


class DomainModel(BaseModel):
    """域名"""
    id = IntegerField(primary_key=True)

    # 域名
    domain = CharField()

    # 别名
    alias = CharField(default="")

    # ip
    ip = CharField(default="")

    # 分组
    group_id = IntegerField(default=0)

    # 签发时间
    start_time = DateTimeField(default=None, null=True)

    # 过期时间
    expire_time = DateTimeField(default=None, null=True)

    # 连接状态
    connect_status = BooleanField(default=False, null=True)

    # 通知状态
    notify_status = BooleanField(default=True)

    # 最后检查时间
    check_time = DateTimeField(default=None, null=True)

    # 详细信息
    detail_raw = TextField(default=None, null=True)

    # 创建时间
    create_time = DateTimeField(default=datetime.now)

    # 更新时间
    update_time = DateTimeField(default=datetime.now)

    @property
    def start_date(self):
        if self.start_time and isinstance(self.start_time, datetime):
            return self.start_time.strftime('%Y-%m-%d')

    @property
    def expire_date(self):
        if self.expire_time and isinstance(self.expire_time, datetime):
            return self.expire_time.strftime('%Y-%m-%d')

    @property
    def total_days(self):
        if self.start_time and self.expire_time:
            return (self.expire_time - self.start_time).days

    @property
    def expire_days(self):
        if self.expire_time:
            return (self.expire_time - datetime.now()).days

    @property
    def detail(self):
        if self.detail_raw:
            return json.loads(self.detail_raw)

    @property
    def group(self):
        if self.group_id:
            return GroupModel.get_or_none(GroupModel.id == self.group_id)



# -*- coding: utf-8 -*-

from flask.json import JSONEncoder as _JSONEncoder

from domain_admin.utils.flask_ext.json.default import default_json_encoder


class JSONEncoder(_JSONEncoder):
    """ flask == 0.1 """

    def default(self, o):
        ret = default_json_encoder(o)
        if ret is not o:
            return ret

        return super().default(o)



# -*- coding: utf-8 -*-
import re
import subprocess
from datetime import datetime

import dateparser
import pytz


def get_re_match_result(pattern, string):
    match = re.search(pattern, string)
    if match:
        return match.group(1)


def parse_params(text):
    data = {}

    for item in text.split(';'):
        [key, value] = item.split('=')
        data[key.strip()] = value

    return data


def parse_time(date_str):
    parsed_time = dateparser.parse(date_str)
    # 解析出来的时间含有时区
    # 转为东8区时间
    return parsed_time.astimezone(pytz.timezone('Asia/Shanghai'))


def format_datetime(date_time):
    return datetime.strftime(date_time, "%Y-%m-%d %H:%M:%S")


def format_date(date_time):
    return datetime.strftime(date_time, "%Y-%m-%d")


def get_cert_info(domain):
    """获取证书信息"""
    cmd = f"curl -Ivs https://{domain} --connect-timeout 10"

    exitcode, output = subprocess.getstatusoutput(cmd)
    # print(exitcode)
    # print(output)

    if exitcode != 0:
        raise Exception('域名连接错误')

    # 正则匹配
    problem = get_re_match_result('SSL certificate problem: (.*)', output)
    if problem:
        raise Exception(problem)

    ip = get_re_match_result('Connected to .*? \((.*?)\)', output)
    subject = get_re_match_result('subject: (.*)', output)
    issuer = get_re_match_result('issuer: (.*)', output)
    start_date = get_re_match_result('start date: (.*)', output)
    expire_date = get_re_match_result('expire date: (.*)', output)

    # 解析匹配结果
    start_date = parse_time(start_date)
    # print(start_date)
    expire_date = parse_time(expire_date)

    return {
        'domain': domain,
        'ip': ip,
        'subject': parse_params(subject),
        'issuer': parse_params(issuer),
        'start_date': format_datetime(start_date),
        'expire_date': format_datetime(expire_date)
    }


if __name__ == '__main__':
    print(get_cert_info('www.baidu1.com'))
